@@ -24,7 +24,6 @@
 
 #include <glib/gi18n.h>
 
-#include "vinagre-notebook.h"
 #include "vinagre-utils.h"
 #include "vinagre-dnd.h"
 #include "vinagre-prefs.h"
@@ -37,6 +36,7 @@ struct _VinagreNotebookPrivate
   GtkUIManager  *manager;
   guint         ui_merge_id;
   VinagreTab    *active_tab;
+  GSList        *tabs;
 };
 
 /* Properties */
@@ -94,6 +94,20 @@ vinagre_notebook_set_property (GObject      *object,
     }
 }
 
+static void
+vinagre_notebook_finalize (GObject *object)
+{
+  VinagreNotebook *nb = VINAGRE_NOTEBOOK (object);
+
+  if (nb->priv->tabs)
+    {
+      g_slist_free (nb->priv->tabs);
+      nb->priv->tabs = NULL;
+    }
+
+  G_OBJECT_CLASS (vinagre_notebook_parent_class)->finalize (object);
+}
+
 static void
 vinagre_notebook_class_init (VinagreNotebookClass *klass)
 {
@@ -101,6 +115,7 @@ vinagre_notebook_class_init (VinagreNotebookClass *klass)
 
   object_class->get_property = vinagre_notebook_get_property;
   object_class->set_property = vinagre_notebook_set_property;
+  object_class->finalize = vinagre_notebook_finalize;
 
   g_object_class_install_property (object_class,
 				   PROP_WINDOW,
@@ -126,9 +141,10 @@ vinagre_notebook_new (VinagreWindow *window)
 void
 vinagre_notebook_show_hide_tabs (VinagreNotebook *nb)
 {
-  gboolean always;
+  gboolean always, fs;
   gint     n;
 
+  fs = vinagre_window_is_fullscreen (nb->priv->window);
   n = gtk_notebook_get_n_pages (GTK_NOTEBOOK (nb));
 
   g_object_get (vinagre_prefs_get_default (),
@@ -136,7 +152,7 @@ vinagre_notebook_show_hide_tabs (VinagreNotebook *nb)
 		NULL);
 
   gtk_notebook_set_show_tabs (GTK_NOTEBOOK (nb),
-			      (n > 1) || (always));
+			      ((n > 1) || (always)) && !fs);
 }
 
 static void
@@ -162,7 +178,7 @@ drag_data_get_handl (GtkWidget *widget,
 
 	/*FIXME: Set other properties of the connection*/
 	gtk_selection_data_set (selection_data,
-				selection_data->target,
+				gtk_selection_data_get_target (selection_data),
 				8,
 				(guchar*) data,
 				strlen (data));
@@ -195,6 +211,27 @@ vinagre_notebook_update_ui_sentitivity (VinagreNotebook *nb)
   active = (nb->priv->active_tab) &&
 	   (vinagre_tab_get_state (VINAGRE_TAB (nb->priv->active_tab)) == VINAGRE_TAB_STATE_CONNECTED);
   gtk_action_group_set_sensitive (action_group, active);
+
+  if (nb->priv->active_tab)
+    {
+      GtkWidget *spinner, *icon;
+
+      spinner = g_object_get_data (G_OBJECT (nb->priv->active_tab), "spinner");
+      icon = g_object_get_data (G_OBJECT (nb->priv->active_tab), "icon");
+
+      if (vinagre_tab_get_state (VINAGRE_TAB (nb->priv->active_tab)) == VINAGRE_TAB_STATE_CONNECTED)
+	{
+	  gtk_widget_hide (spinner);
+	  gtk_spinner_stop (GTK_SPINNER (spinner));
+	  gtk_widget_show (icon);
+	}
+      else
+	{
+	  gtk_widget_hide (icon);
+	  gtk_widget_show (spinner);
+	  gtk_spinner_start (GTK_SPINNER (spinner));
+	}
+    }
 }
 
 static void
@@ -341,11 +378,24 @@ vinagre_notebook_page_switched (GtkNotebook     *notebook,
   vinagre_notebook_update_ui_sentitivity (nb);
 }
 
+static void
+vinagre_notebook_page_removed_cb (VinagreNotebook *nb)
+{
+  gint n;
+
+  if ((gtk_notebook_get_n_pages (GTK_NOTEBOOK (nb)) == 0) &&
+      (vinagre_window_is_fullscreen (nb->priv->window)))
+    vinagre_window_toggle_fullscreen (nb->priv->window);
+
+vinagre_notebook_show_hide_tabs (nb);
+}
+
 static void
 vinagre_notebook_init (VinagreNotebook *nb)
 {
   nb->priv = VINAGRE_NOTEBOOK_GET_PRIVATE (nb);
   nb->priv->active_tab = NULL;
+  nb->priv->tabs = NULL;
 
   gtk_notebook_set_scrollable (GTK_NOTEBOOK (nb), TRUE);
 
@@ -355,7 +405,7 @@ vinagre_notebook_init (VinagreNotebook *nb)
 		    NULL);
   g_signal_connect (nb,
 		    "page-removed",
-		    G_CALLBACK (vinagre_notebook_show_hide_tabs),
+		    G_CALLBACK (vinagre_notebook_page_removed_cb),
 		    NULL);
   g_signal_connect (nb,
 		    "switch-page",
@@ -378,10 +428,10 @@ close_button_clicked_cb (GtkWidget *widget,
 }
 
 static void
-tab_size_changed_cb (VinagreTab *tab, VinagreNotebook *nb)
+vinagre_notebook_update_tab_tooltip (VinagreNotebook *nb, VinagreTab *tab)
 {
-  char *str;
-  GtkWidget *label;
+  char       *str;
+  GtkWidget  *label;
 
   label = GTK_WIDGET (g_object_get_data (G_OBJECT (tab), "label-ebox"));
   g_return_if_fail (label != NULL);
@@ -392,17 +442,26 @@ tab_size_changed_cb (VinagreTab *tab, VinagreNotebook *nb)
   g_free (str);
 }
 
+static void
+tab_tooltip_changed_cb (GObject *object, GParamSpec *pspec, VinagreNotebook *nb)
+{
+  vinagre_notebook_update_tab_tooltip (nb, VINAGRE_TAB (object));
+}
+
 static void
 tab_disconnected_cb (VinagreTab *tab, VinagreNotebook *nb)
 {
-  gchar *message, *name;
+  gchar *message, *name, *emphasis;
 
   name = vinagre_connection_get_best_name (vinagre_tab_get_conn (tab));
-  message = g_strdup_printf (_("Connection to host <i>%s</i> was closed."),
-			     name);
+  emphasis = g_strdup_printf ("<i>%s</i>", name);
+  /* Translators: %s is a host name or IP address. */
+  message = g_strdup_printf (_("Connection to host %s was closed."),
+			     emphasis);
   vinagre_utils_show_error (_("Connection closed"), message, GTK_WINDOW (nb->priv->window));
   g_free (message);
   g_free (name);
+  g_free (emphasis);
 
   vinagre_notebook_close_tab (nb, tab);
 }
@@ -411,13 +470,15 @@ static void
 tab_auth_failed_cb (VinagreTab *tab, const gchar *msg, VinagreNotebook *nb)
 {
   GString *message;
-  gchar   *name;
+  gchar   *name, *emphasis;
 
   message = g_string_new (NULL);
   name = vinagre_connection_get_best_name (vinagre_tab_get_conn (tab));
 
-  g_string_printf (message, _("Authentication to host <i>%s</i> has failed"),
-		   name);
+  emphasis = g_strdup_printf ("<i>%s</i>", name);
+  /* Translators: %s is a host name or IP address. */
+  g_string_printf (message, _("Authentication for host %s has failed"),
+		   emphasis);
   if (msg)
   	g_string_append_printf (message, " (%s)", msg);
   g_string_append_c (message, '.');
@@ -425,6 +486,7 @@ tab_auth_failed_cb (VinagreTab *tab, const gchar *msg, VinagreNotebook *nb)
   vinagre_utils_show_error (_("Authentication failed"), message->str, GTK_WINDOW (nb->priv->window));
   g_string_free (message, TRUE);
   g_free (name);
+  g_free (emphasis);
 
   vinagre_notebook_close_tab (nb, tab);
 }
@@ -432,7 +494,14 @@ tab_auth_failed_cb (VinagreTab *tab, const gchar *msg, VinagreNotebook *nb)
 static void
 tab_initialized_cb (VinagreTab *tab, VinagreNotebook *nb)
 {
+  VinagreConnection *conn = vinagre_tab_get_conn (tab);
+
   vinagre_notebook_update_ui_sentitivity (nb);
+  vinagre_notebook_update_tab_tooltip (nb, tab);
+  vinagre_notebook_update_window_title (nb);
+
+  if (vinagre_connection_get_fullscreen (conn) && (!vinagre_window_is_fullscreen (nb->priv->window)))
+    vinagre_window_toggle_fullscreen (nb->priv->window);
 }
 
 static GtkWidget *
@@ -441,7 +510,7 @@ build_tab_label (VinagreNotebook *nb,
 {
   GtkWidget *hbox, *label_hbox, *label_ebox;
   GtkWidget *label, *dummy_label;
-  GtkWidget *close_button;
+  GtkWidget *close_button, *spinner;
   GtkRcStyle *rcstyle;
   GtkWidget *image;
   GtkWidget *icon;
@@ -486,8 +555,13 @@ build_tab_label (VinagreNotebook *nb,
 		    G_CALLBACK (close_button_clicked_cb),
 		    tab);
 
-  /* setup site icon, empty by default */
-  icon = gtk_image_new ();
+  /* setup spinner */
+  spinner = gtk_spinner_new();
+  gtk_box_pack_start (GTK_BOX (label_hbox), spinner, FALSE, FALSE, 0);
+
+  /* setup site icon */
+  icon = gtk_image_new_from_icon_name (vinagre_tab_get_icon_name (tab),
+				       GTK_ICON_SIZE_MENU);
   gtk_box_pack_start (GTK_BOX (label_hbox), icon, FALSE, FALSE, 0);
 	
   /* setup label */
@@ -508,13 +582,13 @@ build_tab_label (VinagreNotebook *nb,
   gtk_widget_show (dummy_label);	
   gtk_widget_show (image);
   gtk_widget_show (close_button);
-  gtk_widget_show (icon);
   
   g_object_set_data (G_OBJECT (hbox), "label", label);
   g_object_set_data (G_OBJECT (tab),  "label", label);
   g_object_set_data (G_OBJECT (hbox), "label-ebox", label_ebox);
   g_object_set_data (G_OBJECT (tab),  "label-ebox", label_ebox);
-  g_object_set_data (G_OBJECT (hbox), "icon", icon);
+  g_object_set_data (G_OBJECT (tab), "spinner", spinner);
+  g_object_set_data (G_OBJECT (tab), "icon", icon);
   g_object_set_data (G_OBJECT (hbox), "close-button", close_button);
   g_object_set_data (G_OBJECT (tab),  "close-button", close_button);
 
@@ -546,7 +620,8 @@ vinagre_notebook_add_tab (VinagreNotebook *nb,
   /* Unmerge the UI for the current tab */
   unmerge_tab_ui (nb);
 
-  nb->priv->active_tab = tab;  
+  nb->priv->active_tab = tab;
+  nb->priv->tabs = g_slist_append (nb->priv->tabs, tab);
   
   /* Merge the UI for the new tab */
   merge_tab_ui (nb);
@@ -562,8 +637,8 @@ vinagre_notebook_add_tab (VinagreNotebook *nb,
   vinagre_tab_set_notebook (tab, nb);
 
   g_signal_connect (tab,
-		    "notify::original-width",
-		    G_CALLBACK (tab_size_changed_cb),
+		    "notify::tooltip",
+		    G_CALLBACK (tab_tooltip_changed_cb),
 		    nb);
 
   g_signal_connect (tab,
@@ -619,6 +694,7 @@ vinagre_notebook_close_tab (VinagreNotebook *nb,
   position = gtk_notebook_get_current_page (notebook);
   nb->priv->active_tab = VINAGRE_TAB (gtk_notebook_get_nth_page (notebook,
 								 position));
+  nb->priv->tabs = g_slist_remove (nb->priv->tabs, tab);
 
   /* Merge the UI for the new tab (if one exists) */
   if (nb->priv->active_tab != previous_active_tab)
@@ -663,4 +739,12 @@ vinagre_notebook_get_active_tab (VinagreNotebook *nb)
   return nb->priv->active_tab;
 }
 
+GSList *
+vinagre_notebook_get_tabs (VinagreNotebook *nb)
+{
+  g_return_val_if_fail (VINAGRE_IS_NOTEBOOK (nb), NULL);
+
+  return nb->priv->tabs;
+}
+
 /* vim: set ts=8: */
